@@ -1,5 +1,5 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 from collections import deque
@@ -78,11 +78,14 @@ CREDITS = dict(
         'Alex Carvalho (Brazilian Portuguese)',
         'Juliano de Souza Camargo (Brazilian Portuguese)',
         'Giovanne Menicheli (Brazilian Portuguese)',
+        'Fúlvio Alves (Brazilian Portuguese)',
         'twlvnn (Bulgarian)',
         'Roger VC (Catalan)',
+        'Lukáš Linhart (Czech)',
         'Dingzhong Chen (Simplified Chinese)',
         'Eric-Song-Nop (Simplified Chinese)',
         'Inaha (Simplified Chinese)',
+        'LS-Shandong (Simplified Chinese)',
         'randint (Traditional Chinese)',
         'Zhao Se (Traditional Chinese)',
         'happylittle7 (Traditional Chinese)',
@@ -90,7 +93,9 @@ CREDITS = dict(
         'Philip Goto (Dutch)',
         'Koen Benne (Dutch)',
         'Mikachu (Dutch)',
+        'Danial Behzadi (Persian)',
         'Jiri Grönroos (Finnish)',
+        'Ricky Tigg (Finnish)',
         'Irénée THIRION (French)',
         'Valéry Febvre (French)',
         'Mathieu B. (French)',
@@ -104,6 +109,7 @@ CREDITS = dict(
         'Sear Gasor (German)',
         'Vortex Acherontic (German)',
         'Scrambled777 (Hindi)',
+        'mondstern (Hungarian)',
         'Milo Ivir (Croatian)',
         'Alifiyan Rosyidi (Indonesian)',
         'Alim Satria (Indonesian)',
@@ -112,6 +118,7 @@ CREDITS = dict(
         'dedocc (Italian)',
         'Davide Mora (Italian)',
         'Lili Kurek (Polish)',
+        'Aleksander Warzyniak (Polish)',
         'ssantos (Portuguese)',
         'Ademario Cunha (Portuguese)',
         'SpiralPack 527 (Portuguese)',
@@ -123,6 +130,9 @@ CREDITS = dict(
         'Óscar Fernández Díaz (Spanish)',
         'gallegonovato (Spanish)',
         'Jesper (Swedish)',
+        'PaneradFisk (Swedish)',
+        'Willem Dinkelspiel (Swedish)',
+        'தமிழ்நேரம் (Tamil)',
         'Ege Çelikçi (Turkish)',
         'Sabri Ünal (Turkish)',
         'Volkan Yıldırım (Turkish)',
@@ -242,7 +252,7 @@ class ApplicationWindow(Adw.ApplicationWindow):
         self.builder.add_from_resource('/info/febvre/Komikku/ui/menu/main.xml')
 
         self.css_provider = Gtk.CssProvider.new()
-        Gtk.StyleContext.add_provider_for_display(Gdk.Display.get_default(), self.css_provider, 400)
+        Gtk.StyleContext.add_provider_for_display(Gdk.Display.get_default(), self.css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
 
         self.activity_indicator = Adw.Spinner(
             halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER, width_request=48, height_request=48, visible=False
@@ -280,6 +290,7 @@ class ApplicationWindow(Adw.ApplicationWindow):
         self.application.set_accels_for_action('app.enter-search-mode', ['<Primary>f'])
         self.application.set_accels_for_action('app.fullscreen', ['F11'])
         self.application.set_accels_for_action('app.select-all', ['<Primary>a'])
+        self.application.set_accels_for_action('app.library.update', ['<Primary>r'])
         self.application.set_accels_for_action('app.preferences', ['<Primary>comma'])
         self.application.set_accels_for_action('app.shortcuts', ['<Primary>question'])
         self.application.set_accels_for_action('app.quit', ['<Primary>q', '<Primary>w'])
@@ -526,16 +537,15 @@ available in your region/language."""))
         # Override release notes
         dialog.set_release_notes("""
             <ul>
-                <li>[Card] Added `Unread chapters` badge</li>
-                <li>[Card] Info: Updated Genres icon</li>
-                <li>[Servers] Added Manga Koleji (TR)</li>
-                <li>[Servers] Added Webtoon Hatti (TR)</li>
-                <li>[Servers] ComicExtra (EN): Disabled</li>
+                <li>[Library] Added ability to update with &lt;Ctrl+r&gt; shortcut</li>
+                <li>[Card] Info: Improved display of Genres using badges</li>
+                <li>[Servers] Added MangaBin (EN)</li>
+                <li>[Servers] Added Vortex Scans For Free (EN)</li>
                 <li>[Servers] Flame Comics (EN): Update</li>
-                <li>[Servers] Mangago (EN): Update</li>
-                <li>[Servers] View Comics (EN): Update</li>
-                <li>[Servers] Webtoon TR: Update</li>
-                <li>[L10n] Updated French, Russian, Spanish and Ukrainian translations</li>
+                <li>[Servers] Manhuaus (EN): Update</li>
+                <li>[Servers] Perf Scan (FR): Update</li>
+                <li>[Servers] Remanga (RU): Update</li>
+                <li>[L10n] Added Hungarian translation</li>
             </ul>
             <p>Happy reading.</p>
         """)
