@@ -1,5 +1,5 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 from concurrent.futures import as_completed
@@ -66,7 +66,7 @@ class Downloader(GObject.GObject):
                 chapter_id=chapter_id,
                 status='pending',
                 percent=0,
-                date=datetime.datetime.utcnow(),
+                date=datetime.datetime.now(datetime.UTC),
             ))
             chapters_ids.append(chapter_id)
 
