@@ -1,5 +1,5 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 from gettext import gettext as _
@@ -191,10 +191,10 @@ class LibraryPage(Adw.NavigationPage):
 
             sort_order = Settings.get_default().library_sort_order
             if sort_order == 'latest-read-desc':
-                if manga1.last_read > manga2.last_read:
+                if manga1.last_read.timestamp() > manga2.last_read.timestamp():
                     return -1
 
-                if manga1.last_read < manga2.last_read:
+                if manga1.last_read.timestamp() < manga2.last_read.timestamp():
                     return 1
 
             elif sort_order == 'latest-updated-desc':
@@ -212,10 +212,10 @@ class LibraryPage(Adw.NavigationPage):
 
                     return 0
 
-                if manga1.last_update > manga2.last_update:
+                if manga1.last_update.timestamp() > manga2.last_update.timestamp():
                     return -1
 
-                if manga1.last_update < manga2.last_update:
+                if manga1.last_update.timestamp() < manga2.last_update.timestamp():
                     return 1
 
             elif sort_order == 'alphanum-asc':
