@@ -1,5 +1,5 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 from gettext import gettext as _
@@ -654,7 +654,7 @@ class PreferencesServersSettingsSubPage(Adw.NavigationPage):
                         label = Gtk.Label(hexpand=True)
                         label.set_wrap(True)
                         if self.keyring_helper.is_disabled:
-                            label.add_css_class('dim-label')
+                            label.add_css_class('dimmed')
                             label.set_text(_('System keyring service is disabled. Credential cannot be saved.'))
                             box.append(label)
                         elif not self.keyring_helper.has_recommended_backend:
@@ -664,7 +664,7 @@ class PreferencesServersSettingsSubPage(Adw.NavigationPage):
                                 plaintext_checkbutton.set_child(label)
                                 box.append(plaintext_checkbutton)
                             else:
-                                label.add_css_class('dim-label')
+                                label.add_css_class('dimmed')
                                 label.set_text(_('No keyring backends were found to store credential. Plaintext storage will be used as fallback.'))
                                 box.append(label)
 
@@ -690,8 +690,13 @@ class PreferencesServersSettingsSubPage(Adw.NavigationPage):
             else:
                 switchrow = Adw.SwitchRow()
                 switchrow.set_title(html_escape(server_data['name']))
+                subtitle = []
+                if len(server_data['langs']) == 1 and server_data['langs'][0]:
+                    subtitle.append(LANGUAGES[server_data['langs'][0]])
                 if server_data['is_nsfw'] or server_data['is_nsfw_only']:
-                    switchrow.set_subtitle(_('18+'))
+                    subtitle.append(_('18+'))
+                if subtitle:
+                    switchrow.set_subtitle(' · '.join(subtitle))
                 switchrow.set_sensitive(server_allowed)
                 switchrow.set_active(server_enabled and server_allowed)
                 switchrow.connect('notify::active', self.on_server_activated, server_main_id)
