@@ -1,5 +1,5 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 # Madara – WordPress Theme for Manga
@@ -13,23 +13,25 @@
 # ArazNovel [TR] (disabled)
 # Argos Scan / Argos Comics [PT]
 # Asura Scans [TR]
-# Atikrost [TR]
-# Best Manga [RU]
+# Atikrost [TR] (disabled)
+# Best Manga [RU] (disabled)
 # Fr-Scan (Id frdashscan) [FR]
-# Hunlight Scans [EN]
-# Leomanga [ES] (disabled)
-# Leviatanscans [EN]
+# Hunlight Scans [EN] (disabled)
+# Leomanga [ES]
+# Leviatanscans [EN] (disabled)
 # Manga-Scantrad [FR]
 # Mangas Origines [FR] (disabled)
-# MangaWeebs [EN]
+# MangaWeebs [EN] (disabled)
 # Manhuaus [EN]
 # Manhwa Hentai [EN]
 # Phoenix Fansub [ES] (disabled)
+# Poseidon Scans [FR]
 # Reaperscans [AR/FR/ID/TR]
 # Submanga [ES] (disabled)
 # ToonGod [EN]
 # Toonily [EN]
 # Wakascan [FR] (disabled)
+# Webtoon Hatti [TR]
 
 import datetime
 from gettext import gettext as _
@@ -180,7 +182,7 @@ class Madara(Server):
 
                 if status in ('Completed', 'Terminé', 'Completé', 'Completo', 'Finalizado', 'Concluído', 'Tamamlandı', 'مكتملة', 'Закончена'):
                     data['status'] = 'complete'
-                elif status in ('OnGoing', 'En Cours', 'En cours', 'Updating', 'Devam Ediyor', 'Em Lançamento', 'Em andamento', 'En Emision', 'مستمرة', 'Продолжается', 'Выпускается'):
+                elif status in ('OnGoing', 'En Cours', 'En cours', 'Updating', 'Devam Ediyor', 'Em Lançamento', 'Em andamento', 'En Emision', 'En emisión', 'En Emisión', 'مستمرة', 'Продолжается', 'Выпускается'):
                     data['status'] = 'ongoing'
                 elif status in ('Canceled', 'Cancelada'):
                     data['status'] = 'suspended'
